@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2008..2018 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2008..2021 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,8 +24,8 @@
  * SUCH DAMAGE.
  */
 
-#ident "@(#) Copyright (C)2008..2018 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: db_sqlite2.c,v 1.98 2018/09/18 23:29:47 babolo Exp $"
+#ident "@(#) Copyright (C)2008..2021 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: db_sqlite2.c,v 1.105 2021/09/18 17:58:28 babolo Exp $"
 
 #define BLIN_COMPAT      3
 #define Bpars_COMPAT     3
@@ -35,6 +35,7 @@
 #define PGOBLIN_INTERNAL 1
 #define PGOBLIN_MODULE   pgoblin_module
 
+#include <sys/limits.h>
 #include <sys/types.h>
 #include <sys/event.h>
 #include <sys/time.h>
@@ -53,320 +54,16 @@
 #include <multilar.h>
 #include <mife.h>
 #include "pgoblin.h"
-
-/* expansions to SQLite 2 */
-
-#include <regex.h>
-
-void pgoblin_sqlite2_regexp       __P((sqlite_func *ctx, int argc, const char **argv));
-void pgoblin_sqlite2_quote        __P((sqlite_func *ctx, int argc, const char **argv));
-void pgoblin_sqlite2_quote_html   __P((sqlite_func *ctx, int argc, const char **argv));
-void pgoblin_sqlite2_gettimeofday __P((sqlite_func *ctx, int argc, const char **argv));
-
-#define SQLITE_E  0x000001 /*  E    */
-#define SQLITE_SL 0x000002 /*  \                       */
-#define SQLITE_SQ 0x000004 /*  '                       */
-#define SQLITE_DQ 0x000008 /*  "                       */
-#define SQLITE_SF 0x000010 /*  ,          */
-#define SQLITE_EE 0x000020 /*   E             */
-#define SQLITE_US 0x000040 /*                    */
-#define SQLITE_KY 0x000080 /*             */
-#define SQLITE_NL 0x000100 /*  \0                     */
-#define SQLITE_SS 0x000200 /* \X                          */
-
-void
-/**************************************************************************
- **************************************************************************
- **                                                                      **/
-_pgoblin_sqlite2_quote(sqlite_func *ctx, int argc, const char **argv) { /**
- **                                                                      **
- **************************************************************************
- **************************************************************************/
-    int         nquotes;
-    int         keyword;
-    u_int32_t   flags;
-    ssize_t     len;
-    char       *res;
-    const char *cp;
-    int         i;
-    int         j;
-
-    flags = ~(SQLITE_EE | SQLITE_US) & (u_int32_t)BLIN_I(sqlite_user_data(ctx));
-#   define blin_internal_flags (flags & BLIN_MASK)
-    cp = argv[0];
-    if  (!cp) {
-        /* NULL */
-    } else {
-        len = strlen(cp);
-        for (i = 0, nquotes = 0; i < len; i++) {
-            if  (cp[i] == '\\') {
-                flags |= SQLITE_EE | SQLITE_US;
-                if  (flags & SQLITE_SL) nquotes++;
-            } else if (cp[i] == '\'') {
-                flags |= SQLITE_US;
-                if  (flags & SQLITE_SQ) nquotes++;
-            } else if (cp[i] == '\"') {
-                flags |= SQLITE_US;
-                if  (flags & SQLITE_DQ) nquotes++;
-            } else if (((cp[i] >= 'a') && (cp[i] <= 'z')) || (cp[i] == '_')) {
-            } else if ((cp[i] >= '0') && (cp[i] <= '9')) {
-                if  (i == 0) flags |= SQLITE_US;
-            } else if (cp[i] == 0) {
-                flags |= SQLITE_US | SQLITE_NL;
-                if  (flags & SQLITE_SS) nquotes += 3;
-            } else if ((cp[i] >= 0x08) && (cp[i] <= 0x0D)) {
-                flags |= SQLITE_US;
-                if  (flags & SQLITE_SS) nquotes++;
-            } else {
-                flags |= SQLITE_US;
-        }   }
-        if  (!(~flags & (SQLITE_E | SQLITE_EE))) nquotes++;
-        keyword = 0;
-        if  (flags & SQLITE_KY) {
-            keyword = babolo_testword(pgoblin.sqlkeywords, cp);
-            if  (keyword == UNRESERVED_KEYWORD) keyword = 0;
-        }
-        if  (nquotes || keyword || !(flags & SQLITE_SF) || (flags & SQLITE_US)) {
-            if  (!(res = malloc(len + nquotes + 3))) {
-                ifBLIN_QX0("No mem");
-                sqlite_set_result_error(ctx, "No mem", -1);
-            } else {
-                j = 0;
-                if  (!(~flags & (SQLITE_E | SQLITE_EE))) res[j++] = 'E';
-                if  (flags & SQLITE_SQ) res[j++] = '\'';
-                if  (flags & SQLITE_DQ) res[j++] = '\"';
-                for (i = 0; i < len; i++) {
-                    if  ((cp[i] == '\\') && (flags & SQLITE_SL)) res[j++] = '\\';
-                    if  ((cp[i] == '\'') && (flags & SQLITE_SQ)) res[j++] = '\'';
-                    if  ((cp[i] == '\"') && (flags & SQLITE_DQ)) res[j++] = '\"';
-                    res[j++] = cp[i];
-                }
-                if  (flags & SQLITE_DQ) res[j++] = '\"';
-                if  (flags & SQLITE_SQ) res[j++] = '\'';
-                res[j] = '\0';
-                sqlite_set_result_string(ctx, res, j);
-            }
-        } else {
-            sqlite_set_result_string(ctx, cp, -1);
-    }   }
-#   undef blin_internal_flags
-}
-
-static const u_char subst_in[]    = {
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
- 0,0,5,0, 0,0,4,0, 0,0,0,0, 0,0,0,0,
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 3,0,3,0,
-
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
-
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
- 5,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
-
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,
- 0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0};
-
-static const char *subst_out[] = {
- "\000"  ,"\001"  ,"\002"  ,"\003"  , "\004"  ,"\005"  ,"\006"  ,"\007",
- "\010"  ,"\011"  ,"\012"  ,"\013"  , "\014"  ,"\015"  ,"\016"  ,"\017",
- "\020"  ,"\021"  ,"\022"  ,"\023"  , "\024"  ,"\025"  ,"\026"  ,"\027",
- "\030"  ,"\031"  ,"\032"  ,"\033"  , "\034"  ,"\035"  ,"\036"  ,"\037",
- "\040"  ,"\041"  ,"&quot;","\043"  , "\044"  ,"\045"  ,"&amp;" ,"\047",
- "\050"  ,"\051"  ,"\052"  ,"\053"  , "\054"  ,"\055"  ,"\056"  ,"\057",
- "\060"  ,"\061"  ,"\062"  ,"\063"  , "\064"  ,"\065"  ,"\066"  ,"\067",
- "\070"  ,"\071"  ,"\072"  ,"\073"  , "&lt;"  ,"\075"  ,"&gt;"  ,"\077",
-
- "\100"  ,"\101"  ,"\102"  ,"\103"  , "\104"  ,"\105"  ,"\106"  ,"\107",
- "\110"  ,"\111"  ,"\112"  ,"\113"  , "\114"  ,"\115"  ,"\116"  ,"\117",
- "\120"  ,"\121"  ,"\122"  ,"\123"  , "\124"  ,"\125"  ,"\126"  ,"\127",
- "\130"  ,"\131"  ,"\132"  ,"\133"  , "\134"  ,"\135"  ,"\136"  ,"\137",
- "\140"  ,"\141"  ,"\142"  ,"\143"  , "\144"  ,"\145"  ,"\146"  ,"\147",
- "\150"  ,"\151"  ,"\152"  ,"\153"  , "\154"  ,"\155"  ,"\156"  ,"\157",
- "\160"  ,"\161"  ,"\162"  ,"\163"  , "\164"  ,"\165"  ,"\166"  ,"\167",
- "\170"  ,"\171"  ,"\172"  ,"\173"  , "\174"  ,"\175"  ,"\176"  ,"\177",
-
- "\200"  ,"\201"  ,"\202"  ,"\203"  , "\204"  ,"\205"  ,"\206"  ,"\207",
- "\210"  ,"\211"  ,"\212"  ,"\213"  , "\214"  ,"\215"  ,"\216"  ,"\217",
- "\220"  ,"\221"  ,"\222"  ,"\223"  , "\224"  ,"\225"  ,"\226"  ,"\227",
- "&nbsp;","\231"  ,"\232"  ,"\233"  , "\234"  ,"\235"  ,"\236"  ,"\237",
- "\240"  ,"\241"  ,"\242"  ,"\243"  , "\244"  ,"\245"  ,"\246"  ,"\247",
- "\250"  ,"\251"  ,"\252"  ,"\253"  , "\254"  ,"\255"  ,"\256"  ,"\257",
- "\260"  ,"\261"  ,"\262"  ,"\263"  , "\264"  ,"\265"  ,"\266"  ,"\267",
- "\270"  ,"\271"  ,"\272"  ,"\273"  , "\274"  ,"\275"  ,"\276"  ,"\277",
-
- "\300"  ,"\301"  ,"\302"  ,"\303"  , "\304"  ,"\305"  ,"\306"  ,"\307",
- "\310"  ,"\311"  ,"\312"  ,"\313"  , "\314"  ,"\315"  ,"\316"  ,"\317",
- "\320"  ,"\321"  ,"\322"  ,"\323"  , "\324"  ,"\325"  ,"\326"  ,"\327",
- "\330"  ,"\331"  ,"\332"  ,"\333"  , "\334"  ,"\335"  ,"\336"  ,"\337",
- "\340"  ,"\341"  ,"\342"  ,"\343"  , "\344"  ,"\345"  ,"\346"  ,"\347",
- "\350"  ,"\351"  ,"\352"  ,"\353"  , "\354"  ,"\355"  ,"\356"  ,"\357",
- "\360"  ,"\361"  ,"\362"  ,"\363"  , "\364"  ,"\365"  ,"\366"  ,"\367",
- "\370"  ,"\371"  ,"\372"  ,"\373"  , "\374"  ,"\375"  ,"\376"  ,"\377"};
-
-void
-/*******************************************************************************
- *******************************************************************************
- **                                                                           **/
-_pgoblin_sqlite2_quote_html(sqlite_func *ctx, int argc, const char **argv) { /**
- **                                                                           **
- *******************************************************************************
- *******************************************************************************/
-    int         nquotes;
-    u_int32_t   flags;
-    ssize_t     len;
-    char       *res;
-    const char *cp;
-    const char *q;
-    int         i;
-    int         j;
-
-    flags = (u_int32_t)BLIN_I(sqlite_user_data(ctx));
-#   define blin_internal_flags (flags & BLIN_MASK)
-    cp = argv[0];
-    if  (!cp) {
-        /* NULL */
-    } else {
-        len = strlen(cp);
-        for (i = 0, nquotes = 0; i < len; i++) nquotes += (int)subst_in[((u_char*)cp)[i]];
-        if  (nquotes) {
-            if  (!(res = malloc(len + nquotes + 3))) {
-                ifBLIN_QX0("No mem");
-                sqlite_set_result_error(ctx, "No mem", -1);
-            } else {
-                for (j = 0, i = 0; i < len; i++) {
-                    q = subst_out[((u_char*)cp)[i]];
-                    do  {res[j++] = *q++;} while (*q);
-                }
-                res[j] = '\0';
-                sqlite_set_result_string(ctx, res, j);
-            }
-        } else {
-            sqlite_set_result_string(ctx, cp, -1);
-    }   }
-#   undef blin_internal_flags
-}
-
-void
-/****************************************************************************
- ****************************************************************************
- **                                                                        **/
-__pgoblin_sqlite2_regexp(sqlite_func *ctx, int argc, const char **argv) { /**
- **                                                                        **
- ****************************************************************************
- ****************************************************************************/
-    regex_t    *preg;
-    size_t      elen;
-    char       *err;
-    int         exx  = 0;
-    int         ex   = 0;
-
-#   define blin_internal_flags ((u_int32_t)BLIN_I(sqlite_user_data(ctx)) & BLIN_MASK)
-    if  (!argv[0] || !argv[1]) {
-        /* NULL */
-    } else {
-        if  (!(preg = malloc(sizeof(regex_t)))) {
-            sqlite_set_result_error(ctx, "No mem", -1);
-            exx = -2147203043;
-        } else if ((ex = regcomp(preg, argv[0], REG_EXTENDED | REG_NOSUB))) {
-            exx = -2146484339;
-        }
-        if  (!exx) {
-            ex = regexec(preg, argv[1], 0, NULL, 0);
-        }
-        switch (exx ? exx : ex) {
-        case 0          : sqlite_set_result_int(ctx, 1); break;
-        case REG_NOMATCH: sqlite_set_result_int(ctx, 0); break;
-        case -2147203043:                                break;
-        default         : if  (!(err = malloc((elen = regerror(ex, preg, NULL, 0))))) {
-                        ;     sqlite_set_result_error(ctx, "No mem", -1);
-                        ; } else {
-                        ;     regerror(ex, preg, err, elen);
-                        ;     ifBLIN_QX0("regex %s", err);
-                        ;     sqlite_set_result_error(ctx, err, -1);
-                        ;     free(err);
-    }   }               ; }
-#   undef blin_internal_flags
-}
-
-void
-/*********************************************************************************
- *********************************************************************************
- **                                                                             **/
-_pgoblin_sqlite2_gettimeofday(sqlite_func *ctx, int argc, const char **argv) { /**
- **                                                                             **
- *********************************************************************************
- *********************************************************************************/
-    u_int32_t      flags;
-    struct timeval tp;
-
-    flags = (u_int32_t)BLIN_I(sqlite_user_data(ctx));
-#   define blin_internal_flags (flags & BLIN_MASK)
-    if  (gettimeofday(&tp, NULL)) {
-        ifBLIN_QW0("gettimeofday");
-        sqlite_set_result_error(ctx, strerror(errno), -1);
-    } else {
-        sqlite_set_result_double(ctx, 0.000001 * tp.tv_usec + tp.tv_sec);
-    }
-#   undef blin_internal_flags
-}
-
-__weak_reference(_pgoblin_sqlite2_gettimeofday, pgoblin_sqlite2_gettimeofday);
-__weak_reference(__pgoblin_sqlite2_regexp, _pgoblin_sqlite2_regexp);
-__weak_reference(__pgoblin_sqlite2_regexp, pgoblin_sqlite2_regexp);
-__weak_reference(_pgoblin_sqlite2_quote_html, pgoblin_sqlite2_quote_html);
-__weak_reference(_pgoblin_sqlite2_quote, pgoblin_sqlite2_quote);
+#include "pgoblin3sqlite2.h"
 
 /* SQLite 2 interface to pgoblin */
 
 #define INDX(R, C) (R * rsv->ncol + C)
 
-enum states
-{ bg /*        */
-, bq /* \      */
-, bz /*  \0            */
-, be /*  \             */
-, b1 /*  \[0..7]       */
-, b2 /*  \[0..7][0..7] */
-, bb /*       */
-, bd /* \     */
-, bc /*             */
-, bt /*        */
-, bn /*  \N            */
-, bx
-};
-
 static const char statenames[][3] =
 { "bg", "bq", "bz", "be", "b1", "b2", "bb", "bd", "bc", "bt", "bn", "bx"
 };
 
-typedef struct pgoblin_db_conn {
-    BLIN_flag     flags;
-/*                PGOBLIN_DB_TYPE                                                                    */
-#   define        PGOBLIN_STREAMI 0x010000
-#   define        PGOBLIN_STREAMT 0x020000
-#   define        PGOBLIN_STREAME 0x040000
-#   define        PGOBLIN_ALLOUT  (PGOBLIN_STREAMI | PGOBLIN_STREAMT | PGOBLIN_STREAME)
-#   define        PGOBLIN_COPYIN  0x100000
-#   define        PGOBLIN_COPYST  0x200000
-#   define        PGOBLIN_COPYEN  0x400000
-#   define        PGOBLIN_ALLIN   (PGOBLIN_COPYIN | PGOBLIN_COPYST | PGOBLIN_COPYEN)
-    int           ncol;
-    sqlite_vm    *vm;
-    sqlite       *odf;
-    char         *query;
-    u_char       *semi;                         /*  ,      */
-    int           coln;
-    u_char        reg;
-    enum states   state;
-} pgoblin_db_conn;
-
 struct pgoblin_mode {
     BLIN_flag         flags;                                                        /* Must be first */
 /*                    PGOBLIN_DB_TYPE                                                                */
@@ -667,7 +364,7 @@ shurecon(pgoblin_conn *db) {                                        /**
                                , (void*)(long int)db->flags
                                );
     if  (!!ex) {
-        ifBLIN_QX0("regexp creation");
+        ifBLIN_QX0("regexp creation %d %s", ex, exdecode(ex));
         ERROUT(EX_DATAERR, oxdecode(ex));
     }
     ex = sqlite_create_function( conn->odf
@@ -677,7 +374,7 @@ shurecon(pgoblin_conn *db) {                                        /**
                                , (void*)(long int)((db->flags & BLIN_MASK) | SQLITE_SQ)
                                );
     if  (!!ex) {
-        ifBLIN_QX0("quote_literal creation");
+        ifBLIN_QX0("quote_literal creation %d %s", ex, exdecode(ex));
         ERROUT(EX_DATAERR, oxdecode(ex));
     }
     ex = sqlite_create_function( conn->odf
@@ -689,7 +386,7 @@ shurecon(pgoblin_conn *db) {                                        /**
                                         | SQLITE_DQ | SQLITE_SF | SQLITE_KY
                                )        );
     if  (!!ex) {
-        ifBLIN_QX0("quote_ident creation");
+        ifBLIN_QX0("quote_ident creation %d %s", ex, exdecode(ex));
         ERROUT(EX_DATAERR, oxdecode(ex));
     }
     ex = sqlite_create_function( conn->odf
@@ -699,7 +396,7 @@ shurecon(pgoblin_conn *db) {                                        /**
                                , (void*)(long int)((db->flags & BLIN_MASK) | SQLITE_DQ)
                                );
     if  (!!ex) {
-        ifBLIN_QX0("quote_ident_always creation");
+        ifBLIN_QX0("quote_ident_always creation %d %s", ex, exdecode(ex));
         ERROUT(EX_DATAERR, oxdecode(ex));
     }
     ex = sqlite_create_function( conn->odf
@@ -709,7 +406,7 @@ shurecon(pgoblin_conn *db) {                                        /**
                                , (void*)(long int)(db->flags & BLIN_MASK)
                                );
     if  (!!ex) {
-        ifBLIN_QX0("quote_html creation");
+        ifBLIN_QX0("quote_html creation %d %s", ex, exdecode(ex));
         ERROUT(EX_DATAERR, oxdecode(ex));
     }
     ex = sqlite_create_function( conn->odf
@@ -719,7 +416,17 @@ shurecon(pgoblin_conn *db) {                                        /**
                                , (void*)(long int)(db->flags & BLIN_MASK)
                                );
     if  (!!ex) {
-        ifBLIN_QX0("gettimeofday creation");
+        ifBLIN_QX0("gettimeofday creation %d %s", ex, exdecode(ex));
+        ERROUT(EX_DATAERR, oxdecode(ex));
+    }
+    ex = sqlite_create_function( conn->odf
+                               , "instr"
+                               , 2
+                               , pgoblin_sqlite2_instr
+                               , (void*)(long int)(db->flags & BLIN_MASK)
+                               );
+    if  (!!ex) {
+        ifBLIN_QX0("instr creation %d %s", ex, exdecode(ex));
         ERROUT(EX_DATAERR, oxdecode(ex));
     }
 out:
@@ -832,6 +539,7 @@ putcopy(pgoblin_conn *db, const void *buf, int nby) {               /**
  **********************************************************************/
     u_int32_t        control;
     pgoblin_db_conn *conn = db->conn;
+    u_char           cl;
     int              ex   = 0;
     int              i;
     u_char          *c;
@@ -850,19 +558,20 @@ putcopy(pgoblin_conn *db, const void *buf, int nby) {               /**
     }
     bcopy(buf, b, nby);
     c = p = q = b;
-    blin_stateheader(BLIN_4STA24G, pn);
+    blin_stateheader(BLIN_4STX24G, pn);
     for (; conn->state < bx; q++) {
-        control = automa[conn->state][(q < (b + nby)) ? (class[*q] >> 4) : (Lx >> 4)];
-        blin_statebody( BLIN_4STA24G
+        cl = (q < (b + nby)) ? class[*q] : Lx;
+        control = automa[conn->state][cl >> 4];
+        blin_statebody( BLIN_4STX24G
                       , pn
                       , statenames
                       , clasnames[class[*q]]
                       , (char*)q
-                      , p - c
+                      , b + nby - q
                       , control
                       , conn->state
                       , BLIN_I(q)
-                      , conn->reg
+                      , p - c
                       );
         if  (control & Co) c = p = q;
         if  (control & Cb) conn->reg = class[*q] & 0x07;
@@ -888,7 +597,7 @@ putcopy(pgoblin_conn *db, const void *buf, int nby) {               /**
             conn->semi = NULL;
             ifBLIN_QX4("sqlite_bind#1 [%d]NULL", conn->ncol);
             if  (!!(ex = sqlite_bind(conn->vm, ++(conn->ncol), NULL, 0, 0))) {
-                ifBLIN_QX0("sqlite_bind#1");
+                ifBLIN_QX0("sqlite_bind#1 %d %s", ex, exdecode(ex));
                 ERROUT(-EX_SOFTWARE, oxdecode(ex));
         }   }
         if  (control & Cc) { /*   */
@@ -906,13 +615,13 @@ putcopy(pgoblin_conn *db, const void *buf, int nby) {               /**
                 free(conn->semi);
                 conn->semi = NULL;
                 if  (!!ex) {
-                    ifBLIN_QX0("sqlite_bind#2");
+                    ifBLIN_QX0("sqlite_bind#2 %d %s", ex, exdecode(ex));
                     ERROUT(-EX_SOFTWARE, oxdecode(ex));
                 }
             } else {
                 ifBLIN_QX4("sqlite_bind#3 [%d]%d=%s~", conn->ncol, -1, c);
                 if  (!!(ex = sqlite_bind(conn->vm, ++(conn->ncol), (char*)c, -1, 1))) {
-                    ifBLIN_QX0("sqlite_bind#3");
+                    ifBLIN_QX0("sqlite_bind#3 %d %s", ex, exdecode(ex));
                     ERROUT(-EX_SOFTWARE, oxdecode(ex));
         }   }   }
         if  (control & Cl) { /*   */
@@ -1125,7 +834,7 @@ query(pgoblin_conn *db, pgoblin_mode **rsv, char *qry) {            /**
             ex = sqlite_step(conn->vm, &ncol, NULL, NULL);
             ifBLIN_QX4("sqlite_step %d %s", ex, exdecode(ex));
             if  (ex == SQLITE_ROW || ex == SQLITE_DONE) ex = 0;
-            if  (ex) {
+            if  (!!ex) {
                 ifBLIN_QX0("sqlite_step: %d %s", ex, exdecode(ex));
                 if  (!!finalize(db->flags, conn->vm, 0)) {
                     ifBLIN_QX0("");
@@ -1213,7 +922,7 @@ query(pgoblin_conn *db, pgoblin_mode **rsv, char *qry) {            /**
         conn->flags |= PGOBLIN_STREAMI;
         break;
     case PGOBLIN_CR_OK:
-        while (!ex && conn->query && *conn->query) {
+        while (!ex && !!conn->query && !!*conn->query) {
             if  (!!(ex = compile(db, conn->query, &conn->query, cond))) {
                 ifBLIN_QX0("err %d", ex);
                 goto out;
@@ -1229,7 +938,7 @@ query(pgoblin_conn *db, pgoblin_mode **rsv, char *qry) {            /**
                 if  (ex == SQLITE_DONE) ex = 0;
                 break;
             }
-            if  (ex) {
+            if  (!!ex) {
                 ifBLIN_QX0("sqlite_step: %d %s", ex, exdecode(ex));
                 if  (!!finalize(db->flags, conn->vm, 0)) ifBLIN_QX0("");
                 goto out;
@@ -1240,7 +949,7 @@ query(pgoblin_conn *db, pgoblin_mode **rsv, char *qry) {            /**
         }   }
         break;
     case PGOBLIN_CR_TUPL:
-        while (!ex && conn->query && *conn->query) {
+        while (!ex && !!conn->query && !!*conn->query) {
             const char **serv;
 
             if  (!!(ex = compile(db, conn->query, &conn->query, cond))) {
@@ -1248,14 +957,14 @@ query(pgoblin_conn *db, pgoblin_mode **rsv, char *qry) {            /**
                 goto out;
             }
             if  (!conn->vm) break;
+            if  (!!res) clear(&res);
+            if  (!(res = resv(db))) {
+                ifBLIN_QX0("no RAM");
+                ERROUT(EX_OSERR, ENOMEM);
+            }
             for (; ;) {
                 ex = sqlite_step(conn->vm, &ncol, &row, &serv);
                 ifBLIN_QX4("sqlite_step %d %s", ex, exdecode(ex));
-                if  (!!res) clear(&res);
-                if  (!(res = resv(db))) {
-                    ifBLIN_QX0("no RAM");
-                    ERROUT(EX_OSERR, ENOMEM);
-                }
                 if  (0 > (ex = collect(ncol, ex, serv, res))) {
                     ifBLIN_QX0("collect");
                     goto out;
@@ -1270,11 +979,11 @@ query(pgoblin_conn *db, pgoblin_mode **rsv, char *qry) {            /**
                         res->md->tofree = free;
                         res->flags |= PGOBLIN_MULAR;
                     }
-                    while (row && ncol--) {
+                    while (!!row && ncol--) {
                         char *t;
 
                         t = NULL;
-                        if  (*row && !(t = strdup(*row))) {
+                        if  (!!*row && !(t = strdup(*row))) {
                             ifBLIN_QX0("no mem #7");
                             ERROUT(EX_OSERR, ENOMEM);
                         }
@@ -1309,7 +1018,7 @@ query(pgoblin_conn *db, pgoblin_mode **rsv, char *qry) {            /**
         errno = ENOTSUP;
     }
 out:
-    if  (rsv) *rsv = res; else if (res) clear(&res);
+    if  (!!rsv) *rsv = res; else if (!!res) clear(&res);
     ifBLIN_QX4("- %d", ex);
     return(ex);
 #   undef blin_internal_flags
@@ -1397,14 +1106,17 @@ execute(pgoblin_conn *db, pgoblin_mode **rsv, const void *qp, pgoblin_mode *qin)
                       , pgoblin_db_valinfo(PGOBLIN_Length, qin, i, j)
                       , pgoblin_db_getvalue(qin, i, j)
                       );
-            if  (!!(ex = sqlite_bind( vm
-                                    , j + 1
-                                    , pgoblin_db_getvalue(qin, i, j)
-                                    , pgoblin_db_valinfo(PGOBLIN_Length, qin, i, j)
-                                    + (pgoblin_db_getvalue(qin, i, j) ? 1 : 0)
-                                    , 1
-                )  )                ) {
-                ifBLIN_QX0("sqlite_bind#4");
+            ex = sqlite_bind( vm
+                            , j + 1
+                            , pgoblin_db_getvalue(qin, i, j)
+                            , pgoblin_db_valinfo(PGOBLIN_Length, qin, i, j)
+                            + (pgoblin_db_getvalue(qin, i, j) ? 1 : 0)
+                            , 1
+                            );
+            if  (SQLITE_RANGE == ex) {
+                ifBLIN_QX1("No index %d", j + 1);
+            } else if (!!ex) {
+                ifBLIN_QX0("sqlite_bind#4 %d %s", ex, exdecode(ex));
                 ERROUT(-EX_SOFTWARE, oxdecode(ex));
         }   }
         res = NULL;
@@ -1422,8 +1134,7 @@ execute(pgoblin_conn *db, pgoblin_mode **rsv, const void *qp, pgoblin_mode *qin)
                 if  (0 > (ex = collect(ncol, ex, serv, res))) {
                     ifBLIN_QX0("collect");
                     goto out;
-                }
-            }
+            }   }
             ifBLIN_QX4("sqlite_step %d %s", ex, exdecode(ex));
             if  ((PGOBLIN_CR_OK == (cond & ~PGOBLIN_ALTERQ)) & !!ncol) {
                 ifBLIN_QX1("wait for PGOBLIN_CR_OK but received %d cols", ncol);
@@ -1614,7 +1325,6 @@ getcopy(pgoblin_conn *db, void **buf) {                             /**
         *buf = NULL;
         ex = 0;
     } else {
-        char *c;
         int   l;
         int   k;
 
@@ -1623,30 +1333,21 @@ getcopy(pgoblin_conn *db, void **buf) {                             /**
             if (row[ncol]) l += strlen(row[ncol]) * 2 + 1; else l += 3;
         }
         if  (!(*buf = malloc(l))) {
-            ifBLIN_QX0("no RAM");
-            ERROUT(EX_OSERR, ENOMEM);
+            ifBLIN_QW0("no RAM");
+            ex = -EX_OSERR;
+            goto out;
         }
         k = 0;
         for (int ncol = 0; ncol < res->ncol; ncol++) {
-            if  (!(c = row[ncol])) {
-                (*(char**)buf)[k++]  = '\\';
-                (*(char**)buf)[k++]  = 'N';
-            } else {
-                for (int i = 0; !!c[i]; ++i) {
-                    switch(c[i]) {
-                    case 0x08: (*(char**)buf)[k++] = '\\'; (*(char**)buf)[k++] = 'b' ; break;
-                    case 0x09: (*(char**)buf)[k++] = '\\'; (*(char**)buf)[k++] = 't' ; break;
-                    case 0x0A: (*(char**)buf)[k++] = '\\'; (*(char**)buf)[k++] = 'n' ; break;
-                    case 0x0B: (*(char**)buf)[k++] = '\\'; (*(char**)buf)[k++] = 'v' ; break;
-                    case 0x0C: (*(char**)buf)[k++] = '\\'; (*(char**)buf)[k++] = 'f' ; break;
-                    case 0x0D: (*(char**)buf)[k++] = '\\'; (*(char**)buf)[k++] = 'r' ; break;
-                    case '\\': (*(char**)buf)[k++] = '\\'; (*(char**)buf)[k++] = '\\'; break;
-                    default  : (*(char**)buf)[k++] = c[i];
-            }   }   }
-            (*(char**)buf)[k++] = (ncol < (res->ncol - 1)) ? '\t' : '\n';
+            ex = pgoblin_onecopy(&(*buf)[k], row[ncol], -1, (ncol < (res->ncol - 1)) ? -1 : 0);
+            if  (0 > ex) {
+                ifBLIN_QW0("COPY sqlite2");
+                goto out;
+            }
+            k += ex;
             (*(char**)buf)[k] = 0;
         }
-        ex = strlen(*buf);
+        ex = k;
     }
 out:
     return(ex);
@@ -1757,6 +1458,7 @@ valinfo(int kind, pgoblin_mode *rsv, int row, int col) {            /**
     int ex;
 
 #   define blin_internal_flags (rsv->flags & BLIN_MASK)
+    ifBLIN_QX2("+ %08X %d x %d (%d)", rsv->flags, row, col, kind);
     switch (kind) {
     case PGOBLIN_IsNull:
         if  (rsv) {
@@ -1810,10 +1512,10 @@ valinfo(int kind, pgoblin_mode *rsv, int row, int col) {            /**
         }
         break;
     default:
-        ex = -EX_UNAVAILABLE;
+        ex = INT_MIN;
         errno = ENOTSUP;
     }
-    ifBLIN_QX5("- %d", ex);
+    ifBLIN_QX2("- %d", ex);
     return(ex);
 #   undef blin_internal_flags
 }
@@ -1826,6 +1528,8 @@ getvalue(pgoblin_mode *rsv, int row, int col) {                     /**
  **********************************************************************/
     char *c = NULL;
 
+#   define blin_internal_flags (rsv->flags & BLIN_MASK)
+    ifBLIN_QX2("+ %08X %d x %d", rsv->flags, row, col);
     if  (!valinfo(PGOBLIN_IsNull, rsv, row, col)) {
         if  (rsv->flags & PGOBLIN_VM) {
             ifBLIN_QX0("VM instead of MULAR");
@@ -1837,7 +1541,9 @@ getvalue(pgoblin_mode *rsv, int row, int col) {                     /**
             ifBLIN_QX0("NOTHING instead of MULAR");
             errno = EDOOFUS;
     }   }
+    ifBLIN_QX2("- %s", !c ? "" : c);
     return(c);
+#   undef blin_internal_flags
 }
 
 pgoblin_dbases PGOBLIN_MODULE =
